/*
 * Rubber Spoon: LibSteg 0.4.3
 * by Joel R. Voss
 *
 */

// libSteg 0.4.3 desteg
#include "rs043r.h"
// Need to include rs043.h because it has StegKey()
#include "rs043.h"

// readFile function is useful..
#include "readfile.h"

// Image load, edit, and save
// Only accepts ppm and pgm (24-bit and 8-bit raw pnm files)
#include "image.h"

#include <stdio.h>
#include <stdlib.h>

void usage()
{
	printf("desteg04 stegout.pnm data.txt\n");
}


int main(int argc, char **argv)
{
	printf("Rubber Spoon: LibSteg 0.4.3 Test 2\n");
	if(argc < 3)
	{
		usage();
		return 1;
	}
	BYTE *data = 0;
	struct Image image;
	
	// The Portable aNyMap
	image = loadImage(argv[1]);
	int pixelSize = image.height * image.width * image.bpp / 8;
	
	// SHA Hash Key
	int key[50];
	
	// Secret Key
	StegKey("javantea", key);
	
	// taken from steg() real_size, solve for data_size.
	int dataSize = (pixelSize * 50) / (8 * (key[49]+1));
	
	printf("size: %i, %i\n", pixelSize, dataSize);
	if(image.data == 0 || pixelSize <= 0)
	{
		printf("Error: Data or Portable aNyMap is null.\n");
		free(image.data);
		free(data);
		return 1;
	}
	
	// Secret data decoded
	data = (BYTE *)malloc(dataSize);
	if(data == 0)
	{
		printf("Error: Memory allocation failed.\n");
		return 1;
	}
	
	// Actually Desteg the image.
	desteg(data, dataSize, key, 50, image.data);
	
	// Write the secret data to a file.
	FILE *out_file;
	out_file = fopen(argv[2], "w");
	if(out_file == 0)
	{
		printf("Error: Output file not readable.\n");
		free(image.data);
		free(data);
		return 1;
	}
	fwrite(data, dataSize, 1, out_file);
	fclose(out_file);
	
	free(image.data);
	free(data);
	return 0;
}
